@@ -2,7 +2,7 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.6"
+__version__ = "0.1.8"
 
 """
 
@@ -34,6 +34,7 @@ from subprocess import Popen, PIPE, getoutput
 from time import time
 from types import GeneratorType
 from collections import OrderedDict
+from prov.model import ProvEntity, ProvDocument
 
 
 class Program:
@@ -59,13 +60,13 @@ class Program:
         :param version: Version of the program.
         """
         self.name = name
+        self.cmd = cmd
         self.params = parse_params(params)
         self.param_str = generate_param_str(self.params)
         self.tag = tag
         self.path = path_to_bin
         self.version = version
-        self.cmd = cmd
-        self._getoutput = getoutput("which {}".format(self.name))
+        self._getoutput = getoutput(f"which {self.name}")
         self.found = (
             "command not found" not in self._getoutput and self._getoutput != ""
         )
@@ -108,7 +109,7 @@ class Program:
         self.cmd = cmd
         return cmd
 
-    def add_parameter(self, parameter, _print=True):
+    def add_parameter(self, parameter, _print=False):
         """
         Adds a parameter to the current instance and updates the command.
 
@@ -141,10 +142,13 @@ class Program:
         return run_
 
     def serializer(self):
-        serial_out = self.__dict__
-        key = "sample"
-        if key in serial_out.keys():
-            del serial_out[key]
+        serial_out = self.__dict__.copy()
+        keys = [
+            "sample",
+        ]
+        for key in keys:
+            if key in serial_out.keys():
+                del serial_out[key]
         return serializer(serial_out)
 
 
@@ -209,10 +213,17 @@ class Parameter:
                     self.cmd_string = self.value
 
     def __repr__(self):
-        return "Parameter with command string '{}'".format(self.cmd_string)
+        return f"Parameter with command string '{self.cmd_string}'"
 
     def serializer(self):
-        return self.__dict__
+        serial_out = self.__dict__.copy()
+        keys = [
+            "position",
+        ]
+        for key in keys:
+            if key in serial_out.keys():
+                del serial_out[key]
+        return serial_out
 
 
 class Run:
@@ -229,9 +240,9 @@ class Run:
             program, Program
         )
         self.program = program
-        self.params = program.params
-        self.sample = sample
         self.cmd = self.program.cmd
+        self.params = self.program.params
+        self.sample = sample
 
         # Process status
         self.process = None
@@ -276,7 +287,7 @@ class Run:
     @staticmethod
     def _finished_to_status(finished_status):
         """
-        :bool finished_status: 
+        :bool finished_status:
         :return: String representation of status.
         """
         dict_ = {True: "Finished", False: "Pending"}
@@ -298,16 +309,31 @@ class Run:
         # Declare process and start time
         assert (
             self.program.found
-        ), "Cannot find program {}. Make sure it is on your $PATH.".format(
-            self.program.name
-        )
+        ), f"Cannot find program {self.program.name}. Make sure it is on your $PATH."
         if _print:
             str_ = f"Running program '{self.program.name}'"
             if _sample is not None:
                 str_ += f" for sample {_sample.name}."
             else:
                 str_ += "."
-            str_ += "\nCommand is:\n{}".format(self.program.cmd)
+
+            # Pretty printing of commands
+            split_ = self.program.cmd.split()
+            if len(self.program.cmd) > 80:
+                if len(split_) % 2 == 1:
+                    bin_, *fmt_cmd = split_
+                    last = ""
+                else:
+                    bin_, *fmt_cmd, last = split_
+                it = iter(fmt_cmd)
+                fmt_cmd = zip(it, it)
+                fmt_cmd = " \\ \n".join(
+                    [bin_] + ["\t" + i[0] + "\t" + i[1] for i in fmt_cmd] + [last]
+                )
+                str_ += f"\nCommand is:\n{fmt_cmd}"
+            else:
+                str_ += f"\nCommand is:\n{self.program.cmd}"
+
             print(str_)
 
         p = Popen(self.program.cmd, shell=True, stdout=PIPE, stderr=PIPE)
@@ -317,6 +343,10 @@ class Run:
 
         # Run process
         (self.stdout, self.stderr) = p.communicate()
+        self.stdout, self.stderr = (
+            self.stdout.decode("utf-8"),
+            self.stderr.decode("utf-8"),
+        )
 
         # Update status
         end = time()
@@ -338,7 +368,7 @@ class Run:
     def serializer(self):
         # The following lines prevent RecursionError
         serial_out = dict(self.__dict__)
-        for key in ("stdout", "program", "sample"):
+        for key in ("stdout", "program", "sample", "params"):
             if key in serial_out.keys():
                 if key == "stdout":
                     if (
@@ -412,9 +442,7 @@ class PresetProgram(Program):
                 file_ = self.sample.files[tag]
             except KeyError:
                 raise Exception(
-                    "Key '{}' not found in files dictionary of sample '{}':\n'{}'".format(
-                        tag, self.sample.name, self.sample.files
-                    )
+                    f"Key '{tag}' not found in files dictionary of sample '{self.sample.name}':\n'{self.sample.files}'"
                 )
 
             # If in sample, check if it exists
@@ -440,9 +468,8 @@ class PresetProgram(Program):
                 preffix, _ = path.splitext(str(self.sample.files[self.preffix_tag]))
             except KeyError:
                 raise Exception(
-                    "Key '{}' not found in files dictionary of sample '{}':\n'{}'".format(
-                        self.preffix_tag, self.sample.name, self.sample.files
-                    )
+                    f"Key '{self.preffix_tag}' not found in files dictionary of sample '{self.sample.name}':\n"
+                    f"'{self.sample.files}'"
                 )
         try:
             for key, (tag, suffix) in self.output_files.items():
@@ -514,7 +541,7 @@ class PresetProgram(Program):
             # Replace file names with place holders.
             if parameter.kind in ("input", "output"):
                 try:
-                    parameter.value = str(self.sample.files["{}".format(parameter.tag)])
+                    parameter.value = str(self.sample.files[f"{parameter.tag}"])
                 except AttributeError:
                     print("Warning: no sample associated with program.")
                     pass  # Suppress bug for now.
@@ -618,9 +645,7 @@ def add_programs(object_, programs):
     # Assert it is adding to correct object
     assert isinstance(
         object_, (Sample, Project)
-    ), "Can't add file to type '{}'. Can only add file to Sample or Project object.".format(
-        type(object_)
-    )
+    ), f"Can't add file to type '{type(object_)}'. Can only add file to Sample or Project object."
 
     # If a single item, make into tuple and assert correct type
     if isinstance(programs, Program):
@@ -710,6 +735,9 @@ class Sample:
         self.attributes = attributes
         self._programs = None
 
+        # This is an attribute used by the src.prov module
+        self.files_namespace_prefix = None
+
     def __repr__(self):
         str_ = f"Sample {self.name} with {len(self.files)} file(s)."
         return str_
@@ -745,7 +773,12 @@ class Sample:
         Custom serializer for Sample class. Serializes runs, programs, and files attributes.
         :return:
         """
-        return serializer(self)
+        serial_out = self.__dict__.copy()
+        keys = ["files_namespace_prefix"]
+        for key in keys:
+            if key in serial_out.keys():
+                del serial_out[key]
+        return serializer(serial_out)
 
     def run_programs(self, _print=True):
         """
@@ -756,7 +789,7 @@ class Sample:
             for _, p in self.programs.items():
                 self._run_program(p, _print=_print)
         else:
-            print("No programs to run for Sample '{}'".format(self.name))
+            print(f"No programs to run for Sample '{self.name}'")
 
     def _run_program(self, program, _print=True):
         """
@@ -830,6 +863,10 @@ class Project:
         # avoid duplicated user names!
         self.envs = {config.user: config.env}
 
+        # PROV attributes
+        self._entity = None
+        self._document = None
+
     def __len__(self):
         return len(self._samples)
 
@@ -851,6 +888,26 @@ class Project:
     def __setitem__(self, key, value):
         self._samples[key] = value
 
+    @property
+    def entity(self):
+        if self._entity is None:
+            self._entity = ProvEntity(self._document, identifier=f"project:{self}")
+        return self._entity
+
+    @entity.setter
+    def entity(self, value):
+        self._entity = value
+
+    @property
+    def document(self):
+        if self._document is None:
+            self._document = ProvDocument()
+        return self._document
+
+    @document.setter
+    def document(self, document):
+        self._document = document
+
     def keys(self):
         return self._samples.keys()
 
@@ -883,7 +940,7 @@ class Project:
         if sample.name is None:
             slug = generate_slug(2)
             sample.name = slug
-            print("No sample name set. Setting random name: {}".format(sample.name))
+            print(f"No sample name set. Setting random name: {sample.name}")
 
         return sample
 
@@ -994,9 +1051,7 @@ def add_files(object_, files):
     # Assert it is adding to correct object
     assert isinstance(
         object_, (Sample, Project)
-    ), "Can't add file to type '{}'. Can only add file to Sample or Project object.".format(
-        type(object_)
-    )
+    ), f"Can't add file to type '{type(object_)}'. Can only add file to Sample or Project object."
 
     # If it is a dict, we convert to File instances
     if isinstance(files, dict):
@@ -1166,6 +1221,7 @@ def json_to_dict(json_file):
     return dict_
 
 
+# this function is hideous. it must be improved.
 def dict_to_sample(json_dict):
     """
     Converts a JSON dictionary to a sample instance.
@@ -1195,6 +1251,7 @@ def dict_to_sample(json_dict):
                                 value[tag] = SeqFile(
                                     path=file["path"], tag=file["tag"],
                                 )
+                                _ = value[tag].generator
                                 if import_records:
                                     for (
                                         seqstats_attr_
@@ -1216,14 +1273,16 @@ def dict_to_sample(json_dict):
             # Create Program instances
             elif attr == "_programs":
                 for tag, program in value.items():
-                    value[tag] = Program()
+                    value[tag] = Program(program["name"])
                     for program_attr_, program_value_ in program.items():
-
                         # Create Parameter attributes
                         if program_attr_ == "params" and program_value_:
-                            parameter = Parameter()
-                            for param_attr_, param_value_ in program_value_.items():
-                                setattr(parameter, param_attr_, param_value_)
+                            for key, param in program_value_.items():
+                                parameter = Parameter()
+                                for param_attr_, param_value_ in param.items():
+                                    setattr(parameter, param_attr_, param_value_)
+
+                                value[tag].add_parameter(parameter)
 
                         # Create Run instances
                         if program_attr_ == "_runs" and program_value_:
