@@ -2,7 +2,7 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.6"
+__version__ = "0.1.8"
 
 
 """
@@ -66,7 +66,7 @@ def test_Program():
     some_random_program = Program(slug)
     assert (
         not some_random_program.found
-    ), "You shouldn't have a program called {} lying around!".format(slug)
+    ), f"You shouldn't have a program called {slug} lying around!"
 
 
 def test_Parameter():
@@ -273,8 +273,6 @@ def test_project_json_and_prov(debug=False):
     prov_json_out = "./gentax_picocyano_prov.json"
     export_prov_json(prov_json_out, prov)
 
-    # breakpoint()
-
     # Clean up
     for f in (json_out, json_out_2, prov_json_out):
         remove(f)
